
      #include "ladspa-util.h"
      #define MAX_OFFSET 24000
      #define BUFFER_LEN (MAX_OFFSET*2 + 1)
      #define BUFFER_POS(a) (((a) + BUFFER_LEN) % BUFFER_LEN)
      #define MAX(a, b) ((a) > (b) ? (a) : (b))
      #define MIN(a, b) ((a) < (b) ? (a) : (b))
    
#include <math.h>
#include <stdlib.h>
#include "lv2.h"

typedef struct _Offset {
  float *offset;
  float *automatable;
  float *input;
  float *output;
  float *latency;
float * buffer;
long buffer_w_pos;
long last_offset;
} Offset;

static void cleanupOffset(LV2_Handle instance)
{
Offset *plugin_data = (Offset *)instance;

      free(plugin_data->buffer);
    
  free(instance);
}

static void connectPortOffset(LV2_Handle instance, uint32_t port, void *data)
{
  Offset *plugin = (Offset *)instance;

  switch (port) {
  case 0:
    plugin->offset = data;
    break;
  case 1:
    plugin->automatable = data;
    break;
  case 2:
    plugin->input = data;
    break;
  case 3:
    plugin->output = data;
    break;
  case 4:
    plugin->latency = data;
    break;
  }
}

static LV2_Handle instantiateOffset(const LV2_Descriptor *descriptor,
            double s_rate, const char *path,
            const LV2_Feature *const *features)
{
  Offset *plugin_data = (Offset *)malloc(sizeof(Offset));
  float * buffer = plugin_data->buffer;
  long buffer_w_pos = plugin_data->buffer_w_pos;
  long last_offset = plugin_data->last_offset;
  
      buffer = calloc(BUFFER_LEN, sizeof(float));
    
  plugin_data->buffer = buffer;
  plugin_data->buffer_w_pos = buffer_w_pos;
  plugin_data->last_offset = last_offset;
  
  return (LV2_Handle)plugin_data;
}


static void activateOffset(LV2_Handle instance)
{
  Offset *plugin_data = (Offset *)instance;
  float * buffer __attribute__ ((unused)) = plugin_data->buffer;
  long buffer_w_pos __attribute__ ((unused)) = plugin_data->buffer_w_pos;
  long last_offset __attribute__ ((unused)) = plugin_data->last_offset;
  
      memset(buffer, 0, BUFFER_LEN * sizeof(float));
      plugin_data->buffer_w_pos = 0;
      plugin_data->last_offset = 0;
    
}


static void runOffset(LV2_Handle instance, uint32_t sample_count)
{
  Offset *plugin_data = (Offset *)instance;

  const float offset = *(plugin_data->offset);
  const float automatable = *(plugin_data->automatable);
  const float * const input = plugin_data->input;
  float * const output = plugin_data->output;
  float latency;
  float * buffer = plugin_data->buffer;
  long buffer_w_pos = plugin_data->buffer_w_pos;
  long last_offset = plugin_data->last_offset;
  

      long buffer_r_pos, offset_delta, i;

      if (automatable > 0) {

        // report the maximum offset as latency to the host
        *(plugin_data->latency) = MAX_OFFSET;

        // and delay by the maximum offset by default
        // (results in no audible offset)
        buffer_r_pos = BUFFER_POS(buffer_w_pos - MAX_OFFSET);

        // apply user-configured offset (results in the desired offset):
        buffer_r_pos = BUFFER_POS(buffer_r_pos + (long)offset);

      } else {
        // report the currently required latency
        // (the LV2 specification does not allow a negative latency)
        *(plugin_data->latency) = MAX((long)offset, 0);

        // if delaying, apply the configured offset
        // (otherwise we rely on the host's latency compensation)
        buffer_r_pos = BUFFER_POS(buffer_w_pos + MIN((long)offset, 0));
      }

      // if the offset increased, zero the buffer accordingly
      offset_delta = (long)offset - last_offset;
      for (; offset_delta>0; offset_delta--) {
        buffer[BUFFER_POS(buffer_r_pos - offset_delta)] = 0.0f;
      }

      for (i=0; i<sample_count; i++) {
        buffer[buffer_w_pos] = input[i];
        buffer_w_pos = BUFFER_POS(buffer_w_pos + 1);

        buffer_write(output[i], buffer[buffer_r_pos]);
        buffer[buffer_r_pos] = 0.0f;
        buffer_r_pos = BUFFER_POS(buffer_r_pos + 1);
      }

      plugin_data->buffer_w_pos = buffer_w_pos;
      plugin_data->last_offset = (long)offset;

    
}

static const LV2_Descriptor offsetDescriptor = {
  "http://plugin.org.uk/swh-plugins/offset",
  instantiateOffset,
  connectPortOffset,
  activateOffset,
  runOffset,
  NULL,
  cleanupOffset,
  NULL
};


LV2_SYMBOL_EXPORT
const LV2_Descriptor *lv2_descriptor(uint32_t index)
{
  switch (index) {
  case 0:
    return &offsetDescriptor;
  default:
    return NULL;
  }
}
